import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { ThemesService } from '@progress-chef/interact-themes';
import { BreadCrumbItem } from '@progress/kendo-angular-navigation';
import { SVGIcon, homeIcon } from '@progress/kendo-svg-icons';

const defaultItems: BreadCrumbItem[] = [
  {
    text: 'Home',
    title: 'Home',
    svgIcon: homeIcon,
  },
  {
    text: 'Products',
    title: 'Products',
  },
  {
    text: 'Computer peripherals',
    title: 'Computer peripherals',
  },
  {
    text: 'Keyboards',
    title: 'Keyboards',
  },
  {
    text: 'Gaming keyboards',
    title: 'Gaming keyboards',
  },
];

@Component({
  selector: 'lib-breadcrumb',
  templateUrl: './breadcrumb.component.html',
  styleUrls: ['./breadcrumb.component.scss'],
})
export class BreadcrumbComponent implements OnInit {
  @Input() public items: BreadCrumbItem[] = [...defaultItems];
  @Output() valueChange = new EventEmitter<BreadCrumbItem[]>();
  public homeIcon: SVGIcon = homeIcon;

  constructor(private themesService:ThemesService) {}

  ngOnInit(): void {}

  onItemClick(item: BreadCrumbItem) {
    const index = this.items.findIndex(
      (e: BreadCrumbItem) => e.text === item.text
    );
    this.items = this.items.slice(0, index + 1);
    this.emitValueChange(this.items);
  }

  refreshBreadCrumb() {
    this.items = [...defaultItems];
  }

  emitValueChange(itemsToBeEmit: BreadCrumbItem[]) {
    this.valueChange.emit(itemsToBeEmit);
  }
}
