import { AfterViewInit, Component, EventEmitter, INJECTOR, Input, OnChanges, OnInit, Output, SimpleChanges, ViewChild } from '@angular/core';
import { FormControl } from '@angular/forms';
import { TypographyType } from '../typography/TypographyType';
import { ThemesService } from '@progress-chef/interact-themes';
import { ComboBoxComponent } from '@progress/kendo-angular-dropdowns';

@Component({
  selector: 'lib-dropdown',
  templateUrl: './dropdown.component.html',
  styleUrls: ['./dropdown.component.scss']
})
export class DropdownComponent implements OnInit, OnChanges, AfterViewInit {

  @Input() id: string = "sampleId";
  @Input() label: string = "";
  @Input() data: Array<any> = [];
  @Input() isDisabled: boolean = false;
  @Input() clearButton: boolean = false;
  @Input() fillMode: "flat" | "solid" | "outline" | "none" = "solid";
  @Input() filterable: boolean = true;
  @Input() placeholder: string = "";
  @Input() readonly: boolean = false;
  @Input() rounded: "small" | "medium" | "large" | "full" | "none" = "medium";
  @Input() size: "small" | "medium" | "large" | "none" = "medium";
  @Input() suggest: boolean = true;
  @Input() textField!: string;
  @Input() value!: any;
  @Input() valueField!: string;
  @Input() valuePrimitive: boolean = false;
  @Input() model!: any;
  @Output() modelChange = new EventEmitter<any>();
  @Input() control: FormControl = new FormControl();
  @Input() fontType: TypographyType = 'P_MULISH_14_REGULAR';
  @Input() focus: boolean = false;

  constructor(private themesService:ThemesService) { }
  @ViewChild('comboBox', { static: false })
  public comboBox!: ComboBoxComponent;

  ngOnChanges(changes: SimpleChanges){
    if(changes['isDisabled']) {
      if(changes['isDisabled']?.currentValue) {
        this.control.disable();
      } else {
        this.control.enable();
      }
    }
  }

  ngOnInit(): void {
  }

  ngAfterViewInit() {
    if(this.focus) {
      this.comboBox.focus();
    }
  }

  emitModelChange(ev: any) {
    this.modelChange.emit(ev);
  }

}
