import { AfterViewInit, Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges, ViewChild } from '@angular/core';
import { TypographyType } from '../typography/TypographyType';
import { FormControl } from '@angular/forms';
import { ThemesService } from '@progress-chef/interact-themes';
import { MultiSelectComponent } from '@progress/kendo-angular-dropdowns';

@Component({
  selector: 'lib-multi-select-dropdown',
  templateUrl: './multi-select-dropdown.component.html',
  styleUrls: ['./multi-select-dropdown.component.css']
})
export class MultiSelectDropdownComponent implements OnChanges, AfterViewInit {
  @Input() label: string = "";
  @Input() id: string = "sampleId";
  @Input() data: Array<any> = [];
  @Input() isDisabled: boolean = false;
  @Input() clearButton: boolean = false;
  @Input() fillMode: "flat" | "solid" | "outline" | "none" = "solid";
  @Input() checkboxes: boolean = true;
  @Input() filterable: boolean = true;
  @Input() placeholder: string = "";
  @Input() readonly: boolean = false;
  @Input() rounded: "small" | "medium" | "large" | "full" | "none" = "medium";
  @Input() size: "small" | "medium" | "large" | "none" = "medium";
  @Input() textField!: string;
  @Input() valueField: any;
  @Input() model: any;
  @Input() valuePrimitive: boolean = true;
  @Output() modelChange = new EventEmitter<any>();
  @Input() control: FormControl = new FormControl();
  @Input() fontType: TypographyType = 'P_MULISH_14_REGULAR';
  @Input() focus: boolean = false;
  @Input() autoClose: boolean = false;

  constructor(private themesService:ThemesService) { }

  @ViewChild('multiSelect', { static: false })
  public multiSelect!: MultiSelectComponent;

  ngOnChanges(changes: SimpleChanges){
    if(changes['isDisabled']) {
      if(changes['isDisabled']?.currentValue) {
        this.control.disable();
      } else {
        this.control.enable();
      }
    }
  }

  ngAfterViewInit() {
    if(this.focus) {
      this.multiSelect.focus();
    }
  }

  emitModelChange(ev: any) { 
    this.modelChange.emit(ev);  
  }
}
