import {
  Component,
  Input,
  HostBinding,
  Output,
  EventEmitter,
  AfterViewInit,
} from '@angular/core';
import { trigger, transition, style, animate } from '@angular/animations';
import { ThemesService } from '@progress-chef/interact-themes';

export enum Type {
  success = 'success',
  error = 'error',
  warning = 'warning',
  info = 'info',
  none = 'none',
}
const slideDownAnim = trigger('slideDownAnim', [
  transition(':enter', [
    style({
      opacity: 0,
      transform: 'translateY(-100%) scale(0.98)',
    }),
    animate(
      '300ms ease-out',
      style({
        opacity: 1,
        transform: 'translateY(0%) scale(1)',
      })
    ),
  ]),
  transition(':leave', [
    style({ opacity: 1 }),
    animate('195ms ease-in', style({ opacity: 0 })),
  ]),
]);

@Component({
  selector: 'lib-notification',
  templateUrl: './notification.component.html',
  styleUrls: ['./notification.component.scss'],
  animations: [slideDownAnim],
})
export class NotificationComponent implements AfterViewInit {
  @Input() type: Type = Type.info;
  @Input() timeout = 8000;
  @Output() dismissed = new EventEmitter();

  @HostBinding('@slideDownAnim') slideDownAnim: any;
  @HostBinding('class') get theType() {
    return this.type;
  }

  private timeOutRef: ReturnType<typeof setTimeout> | undefined;

  constructor(private themesService:ThemesService) {}
  
  ngAfterViewInit() {
    const isInfinite = this.timeout < 1;
    if (isInfinite) {
      return;
    }

    this.timeOutRef = setTimeout(this.handleClose, this.timeout * 10000);
  }

  handleClose = (): void => {
    clearTimeout(this.timeOutRef);
    this.dismissed.emit();
  };
}
