import {
  AfterViewInit,
  Component,
  EventEmitter,
  Input,
  Output,
  ViewChild,
} from '@angular/core';
import { FormControl } from '@angular/forms';
import { ThemesService } from '@progress-chef/interact-themes';
import { AutoCompleteComponent } from '@progress/kendo-angular-dropdowns';
import { TextBoxComponent } from '@progress/kendo-angular-inputs';
import { map, tap } from 'rxjs';

@Component({
  selector: 'lib-search-box',
  templateUrl: './search-box.component.html',
  styleUrls: ['./search-box.component.scss'],
})
export class SearchBoxComponent implements AfterViewInit {
  @ViewChild('autocomplete', { static: false })
  public autocomplete!: AutoCompleteComponent;
  @ViewChild('textbox', { static: false })
  public textbox!: TextBoxComponent;

  @Input() source: Array<string> = [];
  @Input() data: Array<string> = [];
  @Input() filterable: boolean = true;
  @Input() size: 'small' | 'medium' | 'large' | 'none' = 'small';
  @Input() rounded: "small" | "medium" | "large" | "full" | "none" = "medium";
  @Input() fillMode: "solid" | "flat" | "outline" | "none" = "outline";
  @Input() clearButton: boolean = true;
  @Input() icon: string = 'search';
  @Input() autoComplete: boolean = true;
  @Input() placeholder: string = '';
  @Input() width: number = 200;
  @Input() suggest: boolean = false;
  @Output() modelChange = new EventEmitter<string>();
  @Output() searchClick = new EventEmitter();
  @Input() control: FormControl = new FormControl();
  @Input() value = "";

  constructor(private themesService:ThemesService) { }

  ngAfterViewInit() {
    if (this.autoComplete) {
      const contains = (searchTerm: string) => (item: string) =>
        item.toLowerCase().indexOf(searchTerm.toLowerCase()) !== -1;

      this.autocomplete.filterChange
        .asObservable()
        .pipe(
          tap(() => (this.autocomplete.loading = true)),
          map((searchTerm: string) => this.source.filter(contains(searchTerm)))
        )
        .subscribe((data: string[]) => {
          this.data = data;
          this.autocomplete.loading = false;
        });
    }
  }

  emitValueChange(ev: string) {
    this.modelChange.emit(ev);
  }

  onSearchClick() {
    this.searchClick.emit();
  }
}
