import {
  Component,
  Input,
  EventEmitter,
  Output,
  ViewChild,
  AfterViewInit,
  OnChanges,
  SimpleChanges,
} from '@angular/core';
import { FormControl } from '@angular/forms';
import { ThemesService } from '@progress-chef/interact-themes';
import { TextBoxComponent } from '@progress/kendo-angular-inputs';
@Component({
  selector: 'lib-textbox',
  templateUrl: './textbox.component.html',
  styleUrls: ['./textbox.component.css'],
})
export class TextboxComponent implements AfterViewInit, OnChanges {
  @ViewChild('textbox', { static: false })
  public textbox!: TextBoxComponent;
  showPassword: boolean = false;

  @Input() showEyeIcon: boolean = false;
  @Input() clearButton: boolean = true;
  @Input() placeholder: string = 'message';
  @Input() width: string = '300';
  @Input() isDisabled: boolean = false;
  @Input() readonly: boolean = false;
  @Input() size: "small" | "medium" | "large" = "medium";
  @Input() rounded: "small" | "medium" | "large" | "full" | "none" = "medium";
  @Input() fillMode: "solid" | "flat" | "outline" | "none" = "outline";
  @Input() control: FormControl = new FormControl();
  @Input() value: string = '';
  @Input() type: string =  '';
  @Output() inputChange = new EventEmitter<String>();
  @Output() focusEvent = new EventEmitter();
  @Output() blurEvent = new EventEmitter();

  constructor(private themesService:ThemesService) { }

  ngOnChanges(changes: SimpleChanges){
    if(changes['isDisabled']) {
      if(changes['isDisabled']?.currentValue) {
        this.control.disable();
      } else {
        this.control.enable();
      }
    }
  }

  public ngAfterViewInit(): void {
    this.textbox.input.nativeElement.type = this.type;
  }

  public toggleVisibility(): void {
    const inputElement = this.textbox.input.nativeElement;
    if (inputElement.type === "password") {
      inputElement.type = "text";
      this.showPassword = true;
    } else {
      inputElement.type = "password";
      this.showPassword = false;
    }
  }

  emitValueChange(ev: String) {
    this.inputChange.emit(ev);
  }

  onFocus() {
    this.focusEvent.emit();
  }

  onBlur() {
    this.blurEvent.emit();
  }
}

