import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { FormControl } from '@angular/forms';
import { ThemesService } from '@progress-chef/interact-themes';

@Component({
  selector: 'lib-timescheduler',
  templateUrl: './timescheduler.component.html',
  styleUrls: ['./timescheduler.component.scss'],
})
export class TimeschedulerComponent implements OnInit, OnChanges {
  @Input() model: Date = new Date()
  @Output() modelChange = new EventEmitter<Date>();
  @Input() size: 'small' | 'medium' | 'large' | 'none' = 'medium';
  @Input() rounded: 'small' | 'medium' | 'large' | 'full' | 'none' = 'medium';
  @Input() fillMode: 'solid' | 'flat' | 'outline' | 'none' = 'solid';
  @Input() format: string = 'MM/dd/yyyy hh:mm a';
  @Input() isDisabled: boolean = false;
  @Input() readonly: boolean = false;
  @Input() placeholder = '';
  @Input() control: FormControl = new FormControl();

  constructor(private themesService:ThemesService) { }

  ngOnChanges(changes: SimpleChanges){
    if(changes['isDisabled']) {
      if(changes['isDisabled']?.currentValue) {
        this.control.disable();
      } else {
        this.control.enable();
      }
    }
  }

  ngOnInit(): void { }

  emitModelChange(ev: Date) {
    this.modelChange.emit(ev);
  }
}
