import { AfterContentChecked, AfterContentInit, AfterViewChecked, AfterViewInit, Component, ElementRef, Input, OnChanges, OnInit, ViewChild } from '@angular/core';
import { TypographyType } from './TypographyType';
import { ThemesService } from '@progress-chef/interact-themes';

const Typography = {
  // h1 styles
  H1_MULISH_36_BOLD: 'h1-mulish-36-bold',
  H1_MULISH_36_REGULAR: 'h1-mulish-36-regular',
  H1_MULISH_36_LIGHT: 'h1-mulish-36-light',
  // h2 styles
  H2_MULISH_24_BOLD: 'h2-mulish-24-bold',
  H2_MULISH_24_REGULAR: 'h2-mulish-24-regular',
  H2_MULISH_24_LIGHT: 'h2-mulish-24-light',
  // h3 styles
  H3_MULISH_18_BOLD: 'h3-mulish-18-bold',
  H3_MULISH_18_REGULAR: 'h3-mulish-18-regular',
  H3_MULISH_18_LIGHT: 'h3-mulish-18-light',
  // h4, h5 and h6 styles
  H4_MULISH_14_BOLD: 'h4-mulish-14-bold',
  H5_MULISH_14_REGULAR: 'h5-mulish-14-regular',
  H6_MULISH_14_LIGHT: 'h6-mulish-14-light',
  // p 16px styles
  P_MULISH_16_BOLD: 'p-mulish-16-bold',
  P_MULISH_16_REGULAR: 'p-mulish-16-regular',
  P_MULISH_16_REGULAR_ITALIC: 'p-mulish-16-regular-italic',
  P_MULISH_16_LIGHT: 'p-mulish-16-light',
  P_MULISH_16_LIGHT_ITALIC: 'p-mulish-16-light-italic',
  // p 14px styles
  P_MULISH_14_BOLD: 'p-mulish-14-bold',
  P_MULISH_14_REGULAR: 'p-mulish-14-regular',
  P_MULISH_14_REGULAR_ITALIC: 'p-mulish-14-regular-italic',
  P_MULISH_14_LIGHT: 'p-mulish-14-light',
  P_MULISH_14_LIGHT_ITALIC: 'p-mulish-14-light-italic',
  // small 12px styles
  SMALL_MULISH_12_BOLD: 'small-mulish-12-bold',
  SMALL_MULISH_12_REGULAR: 'small-mulish-12-regular',
  SMALL_MULISH_12_REGULAR_ITALIC: 'small-mulish-12-regular-italic',
  SMALL_MULISH_12_LIGHT: 'small-mulish-12-light',
  SMALL_MULISH_12_LIGHT_ITALIC: 'small-mulish-12-light-italic',
  // Custom
  BASE_TEXT_MULISH: 'base-text-mulish'
}

@Component({
  selector: 'lib-typography',
  templateUrl: './typography.component.html',
  styleUrls: ['./typography.component.scss']
})

export class TypographyComponent implements AfterViewInit, AfterViewChecked, OnChanges {
  @Input() type: TypographyType = 'P_MULISH_14_REGULAR';
  @Input() defaultText: string = "defaultText";
  @Input() isLabel: boolean = false;
  @Input() hiddenLabel: string = "";
  @ViewChild('ref', { static: false }) child: ElementRef | undefined;
  @ViewChild('ref_label', { static: false }) labelChild: ElementRef | undefined;
  showDefaultText = true;
  font = Typography[this.type];
  input_id = null;

  constructor(private themesService:ThemesService) { }
  
  ngAfterViewChecked(): void {
    // this.timeOutRef = setTimeout(this.checkForLabel, 15000);
    if(!this.input_id) {
      this.checkForLabel()
    }
  }

  ngOnChanges() {
    this.font = Typography[this.type];
  }

  ngAfterViewInit() {
    this.checkShowDefaultText();
  }

  checkForLabel() {

    let childrenCount = this.labelChild?.nativeElement.childElementCount;
    if(childrenCount > 0) {
      this.input_id = this.labelChild?.nativeElement.getElementsByTagName('input')[0]?.getAttribute('id');
    }
  }

  checkShowDefaultText() {
    let childrenCount = this.child?.nativeElement.childNodes.length;
    this.showDefaultText = !this.isLabel && (!childrenCount || childrenCount == 0);
  }
}
