import {
  ChangeDetectorRef, Component, ElementRef, Input, OnInit, Renderer2, ViewChild
} from '@angular/core';

@Component({
  selector: 'lib-dialog',
  templateUrl: './dialog.component.html',
  styleUrls: ['./dialog.component.css'],
})
export class DialogComponent implements OnInit {
  @ViewChild('contentRef', { static: false }) contentRef:
    | ElementRef
    | undefined;
  @ViewChild('dialogRef', { static: false }) dialogRef: ElementRef | undefined;

  @Input() width!: string | number;
  @Input() height!: string | number;
  @Input() minWidth!: string | number;
  @Input() maxWidth!: string | number;
  @Input() minHeight!: string | number;
  @Input() maxHeight!: string | number;
  @Input() public opened: true | false = true;

  constructor(
    private elt: ElementRef,
    private ref: ChangeDetectorRef,
    private renderer: Renderer2
  ) { }

  ngOnInit(): void { }

  ngAfterViewInit() {
    //detect changes
    this.ref.detectChanges();

    // swap the node positions in ng-content to match the original kendo dom structure
    const contentRefElement = this.contentRef?.nativeElement;
    const dialogRefElement = this.dialogRef?.nativeElement;
    const libTitleBar = contentRefElement.getElementsByTagName(
      'lib-dialog-titlebar'
    );
    const libActionsBar =
      contentRefElement.getElementsByTagName('lib-dialog-actions');
    const parentDialogElement =
      dialogRefElement.getElementsByClassName('k-window');
    this.renderer.insertBefore(
      parentDialogElement[0],
      libTitleBar[0],
      contentRefElement.parentElement
    );
    this.renderer.appendChild(parentDialogElement[0], libActionsBar[0]);
  }
}
