import { AfterViewInit, Component, ElementRef, Input, OnInit, Renderer2 } from '@angular/core';

@Component({
  selector: 'lib-grid-layout-item',
  templateUrl: './grid-layout-item.component.html',
  styleUrls: ['./grid-layout-item.component.css']
})
export class GridLayoutItemComponent implements OnInit, AfterViewInit {

  @Input() rowNumber!: number;
  @Input() columnNumber!: number;
  @Input() rowSpan: number = 1;
  @Input() colSpan: number = 1;

  constructor(
    private renderer: Renderer2,
    private elRef: ElementRef
  ) {

  }

  ngOnInit(): void {
  }

  ngAfterViewInit(): void {
    // access the DOM. get the element to unwrap
    const el = this.elRef.nativeElement; // app-page
    const parent = this.renderer.parentNode(this.elRef.nativeElement); // parent
    // move children to parent (everything is moved including comments which angular depends on)
    while (el.firstChild) { // this line doesn't work with server-rendering
      this.renderer.appendChild(parent, el.firstChild);
    }
    // remove empty element from parent - true to signal that this removed element is a host element
    this.renderer.removeChild(parent, el, true);
  }

}
