import { Component, ContentChild, EventEmitter, Input, OnInit, Output, Renderer2, TemplateRef } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { AddEvent, CancelEvent, EditEvent, GridComponent, RemoveEvent, SaveEvent } from '@progress/kendo-angular-grid';
import { State } from '@progress/kendo-data-query';

const createFormGroup = (controls: any): FormGroup => {
  const form = new FormGroup({});
  for (let controlName of Object.keys(controls)) {
    form.addControl(controlName, new FormControl(controls[controlName], Validators.required))
  }
  return form;
}

@Component({
  selector: 'lib-grid-form',
  templateUrl: './grid-form.component.html',
  styleUrls: ['./grid-form.component.scss']
})
export class GridFormComponent implements OnInit {
  @Input() showToolbar: boolean = false;
  @Input() noRecordsContent: string = 'No tags added yet';
  showColumns = false
  public formGroup!: FormGroup | any;

  @ContentChild('tmp1') templateRef1!: TemplateRef<any> | null;
  @ContentChild('tmp2') templateRef2!: TemplateRef<any> | null;

  @Input() gridData: any[] = [];
  @Input() columnData: any[] = [];
  @Input() controls: any = {};
  counter!: number;
  @Output() addOrUpdateRow = new EventEmitter<any>();
  @Output() deletedRow = new EventEmitter<any>();

  private editedRowIndex: number | undefined;
  @Output() removeRow = new EventEmitter<any>();
  @Output() saveRow = new EventEmitter<any>();

  constructor(private renderer: Renderer2) { }

  ngOnInit(): void {
  }


  public onStateChange(state: State): void {
    console.log('onStateChange: ', state);
  }

  public addHandler(args: AddEvent): void {
    this.showColumns = true
    this.closeEditor(args.sender);
    // define all editable fields validators and default values
    this.formGroup = createFormGroup(this.controls);;
    // show the new row editor, with the `FormGroup` build above
    args.sender.addRow(this.formGroup);
  }

  public editHandler(args: EditEvent): void {
    // define all editable fields validators and default values
    const { dataItem } = args;
    this.closeEditor(args.sender);

    this.formGroup = createFormGroup(dataItem);
    this.editedRowIndex = args.rowIndex;
    // put the row in edit mode, with the `FormGroup` build above
    args.sender.editRow(args.rowIndex, this.formGroup);
  }

  public cancelHandler(args: CancelEvent): void {
    // close the editor for the given row
    this.closeEditor(args.sender, args.rowIndex);
    if(this.gridData.length === 0){
      this.showColumns = false;
    }
  }

  public saveHandler({ sender, rowIndex, formGroup, isNew }: SaveEvent): void {
    this.saveRow.emit({ rowData: formGroup.value, isNew });
    // Close the editor for a given row 
    sender.closeRow(rowIndex);
  }

  public removeHandler(args: RemoveEvent): void {
    // remove the current dataItem from the current data source,
    this.removeRow.emit(args.dataItem);
    if(this.gridData.length === 0){
      this.showColumns = false;
    }
  }

  private closeEditor(grid: GridComponent, rowIndex = this.editedRowIndex) {
    // close the editor
    grid.closeRow(rowIndex);
    // reset the helpers
    this.editedRowIndex = undefined;
    this.formGroup = undefined;
  }

}
