import {
  AfterContentInit,
  AfterViewInit,
  Component,
  ContentChild,
  EventEmitter,
  Input,
  OnChanges,
  Output,
  SimpleChanges,
  TemplateRef,
  ViewChild,
} from '@angular/core';
import {
  SelectableSettings,
  SelectableMode,
  CellClickEvent,
  GridComponent,
} from '@progress/kendo-angular-grid';
import {
  SVGIcon,
  moreVerticalIcon,
  chevronDownIcon,
  chevronUpIcon,
  trashIcon,
} from '@progress/kendo-svg-icons';

@Component({
  selector: 'lib-grid',
  templateUrl: './grid.component.html',
  styleUrls: ['./grid.component.scss'],
})
export class LibGridComponent implements AfterContentInit, OnChanges, AfterViewInit {
  @ContentChild('multiColumnData') templateRef!: TemplateRef<any> | null;
  @ContentChild('detailTemplate') detailTemplate!: TemplateRef<any> | null;
  @ContentChild('impactColumn') impactColumn!: TemplateRef<any> | null;
  @ViewChild(GridComponent) grid!: GridComponent;

  @Input() gridData: any = [];
  @Input() columnData: any = [];
  @Input() pageSize: number = 0;
  @Input() pageable: boolean = false;
  @Input() sortable: boolean = false;
  @Input() groupable: boolean = false;
  @Input() reorderable: boolean = false;
  @Input() resizable: boolean = false;
  @Input() checkBoxWidth: number = 28;
  @Input() checkBoxColumnResizable: boolean = false;
  @Input() checkBoxColumnMenu: boolean = false;
  @Input() checkBoxColumnShowSelectedAll: boolean = true;
  @Input() checkBoxRequired: boolean = false;
  @Input() columnMenu: boolean = false;
  @Input() mode: SelectableMode | undefined = 'multiple';
  @Input() checkboxOnly = true;
  @Input() dragColumns = false;
  @Input() navigable: boolean = true;
  @Input() height!: number;
  @Input() gridRowSelectByValue: string = '';
  @Input() noRecordsContent: string = 'No Records Available';
  @Output() selectedRowData = new EventEmitter<any[]>();
  @Input() showToolbar: boolean = false;
  extractDataItems: any;
  selectableSettings: SelectableSettings = {};
  @Input() kebabList: any = [];
  @Input() showKebab: boolean = false;
  @Output() rowDetails = new EventEmitter<any[]>();
  @Output() kebabItemClick = new EventEmitter<any>();
  @Input() icon: SVGIcon = moreVerticalIcon;
  @Input() kebabColumnWidth: number = 60;
  data: any = [];
  @Output() rowDataOfClickedCell = new EventEmitter<any>();
  @Input() preSelected: any[] = [];
  preSelectedRecords: any[] = [];
  @Input() ignoreColumnCellClick: number[] = [];
  @Input() allowColumnClick: boolean = true;
  public arrowDownIcon: SVGIcon = chevronDownIcon;
  public arrowUpIcon: SVGIcon = chevronUpIcon;
  @Input() expandRowIndex: number[] = [];
  @Input() customExpandRowColumn:boolean = true;
  deleteIcon:SVGIcon = trashIcon;
  @Input() deleteColumnWidth:number = 42;
  @Input() expandRowColumnWidth:number = 42;
  @Output() deletedRowData = new EventEmitter<any>();
  @Input() expandRows: boolean = false;
  @Input() showDeleteColumn = false;

  constructor() {
    this.setSelectableSettings();
  }

  ngOnChanges(changes: SimpleChanges) {
    let prevPreSelected = JSON.stringify(changes['preSelected']?.previousValue);
    let currPreSelected = JSON.stringify(changes['preSelected']?.currentValue);
    // Check if there is a change in this.preSelected input
    if (prevPreSelected !== currPreSelected) {
      // Clear the this.preSelectedRecords array
      this.preSelectedRecords = [];
      for (
        let row = 0;
        row < changes['preSelected'].currentValue.length;
        row++
      ) {
        for (let col = 0; col < this.gridData.length; col++) {
          // Find the relevant record for a particular id
          if (
            this.gridData[col][this.gridRowSelectByValue] ===
            changes['preSelected'].currentValue[row]
          ) {
            let found = false;
            // Check if this record is present in this.data, if yes don't push it in this.preSelectedRecords else push it
            for (let index = 0; index < this.data.length; index++) {
              if (
                this.data[index][this.gridRowSelectByValue] ===
                this.gridData[col][this.gridRowSelectByValue]
              ) {
                found = true;
              }
            }
            if (!found) {
              this.preSelectedRecords.push(this.gridData[col]);
            }
          }
        }
      }
      this.selectedRowData.emit(this.preSelectedRecords.concat(this.data));
    }
  }

  ngAfterContentInit(): void {
    if (!this.pageable) {
      this.pageSize = this.gridData.length;
    }
  }

  ngAfterViewInit() {
    if(this.customExpandRowColumn){
      this.expandRowIndex.forEach((id) => {
        this.grid.expandRow(id);
      });
    }
  }

  setSelectableSettings(): void {
    if (this.checkboxOnly || this.mode === 'single') {
      this.dragColumns = false;
    }

    this.selectableSettings = {
      checkboxOnly: this.checkboxOnly,
      mode: this.mode,
      drag: this.dragColumns,
    };
  }

  // We can extract data out of row number using below function
  /* extract() {
    this.extractDataItems = this.mySelection.map(idx => {
      return this.gridData.find((data: { ProductID: number; }) => data.ProductID === idx);
    });
    console.log(this.extractDataItems)
    this.selectedRowData.emit(this.extractDataItems);
  } */

  onSelectionChange(args: any): any {
    for (let row = 0; row < args.selectedRows.length; row++) {
      var found = false;
      for (let col = 0; col < this.data.length; col++) {
        if (
          this.data[col][this.gridRowSelectByValue] ===
          args.selectedRows[row].dataItem[this.gridRowSelectByValue]
        ) {
          found = true;
        }
      }
      if (!found) {
        this.data.push(args.selectedRows[row].dataItem);
      }
    }
    if (args.deselectedRows.length !== 0) {
      for (let row = 0; row < args.deselectedRows.length; row++) {
        for (let col = 0; col < this.data.length; col++) {
          if (
            this.data[col][this.gridRowSelectByValue] ===
            args.deselectedRows[row].dataItem[this.gridRowSelectByValue]
          ) {
            this.data.splice(col, 1);
          }
        }
        for (let col = 0; col < this.preSelectedRecords.length; col++) {
          if (
            this.preSelectedRecords[col][this.gridRowSelectByValue] ===
            args.deselectedRows[row].dataItem[this.gridRowSelectByValue]
          ) {
            this.preSelectedRecords.splice(col, 1);
            // Remove this element from preselected
            let index = this.preSelected.indexOf(
              args.deselectedRows[row].dataItem[this.gridRowSelectByValue]
            );
            if (index !== -1) {
              this.preSelected.splice(index, 1);
            }
          }
        }
      }
    }
    this.selectedRowData.emit(this.preSelectedRecords.concat(this.data));
  }

  onKebabClick(item: any, rowDetails: any) {
    this.kebabItemClick.emit({ action: item, rowDetails });
  }

  onDetailExpand(rowData: any) {
    this.rowDetails.emit(rowData.dataItem);
  }

  cellClickHandler(args: CellClickEvent) {
    if (this.allowColumnClick) {
      if (this.ignoreColumnCellClick.includes(args.columnIndex)) {
        return;
      }
      this.rowDataOfClickedCell.emit(args.dataItem);
    }
  }

  expandRow(id: number) {
    this.grid.expandRow(id);
    this.expandRowIndex.push(id);
  }

  collapseRow(id: number) {
    this.grid.collapseRow(id);
    this.expandRowIndex.splice(this.expandRowIndex.indexOf(id), 1);
  }

  onDelete(rowDetails: any) {
    this.deletedRowData.emit(rowDetails);
  }
}
