import {
  moduleMetadata,
  Meta,
  StoryObj,
  applicationConfig,
} from '@storybook/angular';
import { PopupModule } from '@progress/kendo-angular-popup';
import { AngularPopupComponent } from '@components/atoms/angular-popup/angular-popup.component';
import { ContainerModule } from '@components/atoms/container/container.module';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<AngularPopupComponent> = {
  title: 'Atoms/Angular Popup Component',
  component: AngularPopupComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      imports: [PopupModule, ContainerModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    show: {
      description: `Provide a boolean value to decide whether to show the popup or not.`,
      table: {
        defaultValue: {
          summary: 'true',
        },
      },
    },
    anchor: {
      description: `To align the Popup to a specific component, use the anchor property binding. The Popup opens next to the defined anchor component.`,
      type: {
        required: true,
        name: 'string',
      },
    },
    anchorHorizontalAlign: {
      options: ['left', 'center', 'right'],
      control: 'radio',
      description:
        'Defines the possible horizontal point values that are relative to the anchor.',
      table: {
        defaultValue: {
          summary: 'right',
        },
      },
    },
    anchorVerticalAlign: {
      options: ['top', 'center', 'bottom'],
      control: 'radio',
      description:
        'Defines the possible vertical point values that are relative to the anchor.',
      table: {
        defaultValue: {
          summary: 'bottom',
        },
      },
    },
    popupHorizontalAlign: {
      options: ['left', 'center', 'right'],
      control: 'radio',
      description:
        'Defines the possible horizontal point values that are relative to the Popup.',
      table: {
        defaultValue: {
          summary: 'right',
        },
      },
    },
    popupVerticalAlign: {
      options: ['top', 'center', 'bottom'],
      control: 'radio',
      description:
        'Defines the possible vertical point values that are relative to the Popup.',
      table: {
        defaultValue: {
          summary: 'top',
        },
      },
    },
  },
  render: (args: AngularPopupComponent) => ({
    props: args,
    template: `
    <button #anchor style="margin:100px">Anchor</button>
    <lib-angular-popup [show]='show' [anchor]="anchor" [anchorHorizontalAlign]="anchorHorizontalAlign" [anchorVerticalAlign]="anchorVerticalAlign" [popupHorizontalAlign]="popupHorizontalAlign" [popupVerticalAlign]="popupVerticalAlign">
      Popup content.
    </lib-angular-popup>
  `,
  }),
};
export default meta;

type Story = StoryObj<AngularPopupComponent>;

export const AngularPopup: Story = {
  args: {
    show: true,
    anchorHorizontalAlign: 'right',
    anchorVerticalAlign: 'bottom',
    popupHorizontalAlign: 'right',
    popupVerticalAlign: 'top',
  },
};
