import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { LayoutModule } from '@progress/kendo-angular-layout';
import { AvatarComponent } from '@components/atoms/avatar/avatar.component';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import '@angular/localize/init';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<AvatarComponent> = {
  title: 'Atoms/AvatarComponent',
  component: AvatarComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      imports: [LayoutModule, TypographyModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    size: {
      description: 'Specify "small" , "medium" or "large"',
      options: ['small', 'medium', 'large', 'none'],
      control: 'select',
    },
    rounded: {
      options: ['small', 'medium', 'large', 'full', 'none'],
      control: 'select',
    },
    fillMode: {
      options: ['solid', 'outline', 'none'],
      control: 'select',
    },
    themeColor: {
      options: [
        'base',
        'primary',
        'secondary',
        'tertiary',
        'info',
        'success',
        'warning',
        'error',
        'dark',
        'light',
        'inverse',
        'none',
      ],
      control: 'select',
    },
  },
  render: (args: AvatarComponent) => ({
    props: args,
  }),
};
export default meta;

type Story = StoryObj<AvatarComponent>;

export const avatar: Story = {
  args: {
    themeColor: 'primary',
    rounded: 'full',
    fillMode: 'solid',
    size: 'small',
    imageURL:
      'https://demos.telerik.com/kendo-ui/content/web/Customers/GOURL.jpg',
    showInitials: true,
    name: 'MFE',
  },
};
