import { moduleMetadata, Meta, StoryObj } from '@storybook/angular';
import { NavigationModule } from '@progress/kendo-angular-navigation';
import { ButtonsModule } from '@progress/kendo-angular-buttons';
import { DropDownsModule } from '@progress/kendo-angular-dropdowns';
import { InputsModule } from '@progress/kendo-angular-inputs';
import { BreadcrumbComponent } from '@components/atoms/breadcrumb/breadcrumb.component';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';
import { homeIcon } from '@progress/kendo-svg-icons';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<BreadcrumbComponent> = {
  title: 'Atoms/BreadcrumbComponent',
  component: BreadcrumbComponent,
  decorators: [
    moduleMetadata({
      imports: [
        ButtonsModule,
        NavigationModule,
        DropDownsModule,
        InputsModule,
        ThemesModule,
      ],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    items: {
      control: 'object',
    },
    emitValueChange: {
      action: true,
    },
  },
  render: (args: BreadcrumbComponent) => ({
    props: args,
  }),
};
export default meta;

type Story = StoryObj<BreadcrumbComponent>;

export const Breadcrumb: Story = {
  args: {
    items: [
      {
        title: 'Home',
        svgIcon: homeIcon,
      },
      {
        text: 'Compliance',
        title: 'Compliance',
      },
      {
        text: 'Scan Job',
        title: 'Scan Job',
      },
      {
        text: 'Create Scan Job',
        title: 'CreateScan Job',
      },
    ],
  },
};
export const breadcrumbInteraction: Story = {
  args: {
    items: [
      {
        title: 'Home',
        svgIcon: homeIcon,
      },
      {
        text: 'Compliance',
        title: 'Compliance',
      },
      {
        text: 'Scan Job',
        title: 'Scan Job',
      },
      {
        text: 'Create Scan Job',
        title: 'CreateScan Job',
      },
    ],
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    const checkShow = await canvas.getByTitle('Scan Job');
    await userEvent.click(checkShow);
  },
};
