import { moduleMetadata, Meta, StoryObj } from '@storybook/angular';
import { ButtonsModule } from '@progress/kendo-angular-buttons';
import { ButtonGroupComponent } from '@components/atoms/button-group/button-group.component';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { IconModule } from '@components/atoms/icon/icon.module';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<ButtonGroupComponent> = {
  title: 'Atoms/ButtonGroupComponent',
  component: ButtonGroupComponent,
  decorators: [
    moduleMetadata({
      imports: [ButtonsModule, TypographyModule, ThemesModule, IconModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    size: {
      description: 'Specify "small" , "medium" or "large"',
      options: ['small', 'medium', 'large'],
      control: 'select',
    },
    rounded: {
      options: ['small', 'medium', 'large', 'full', 'none'],
      control: 'select',
    },
    fillMode: {
      options: ['solid', 'flat', 'outline', 'clear', 'link', 'none'],
      control: 'select',
    },
    themeColor: {
      options: [
        'base',
        'primary',
        'secondary',
        'tertiary',
        'info',
        'success',
        'warning',
        'error',
        'dark',
        'light',
        'inverse',
      ],
      control: 'select',
    },
    disabled: {
      control: 'boolean',
    },
    selection: {
      description: 'Allow single or multiselect',
      options: ['single', 'multiple'],
      control: 'select',
    },
  },
  render: (args: ButtonGroupComponent) => ({
    props: args,
  }),
};
export default meta;

type Story = StoryObj<ButtonGroupComponent>;

export const ButtonGroup: Story = {
  args: {
    buttons: [
      { label: 'First', selected: false },
      { label: 'Second', selected: true },
    ],
    size: 'large',
    rounded: 'small',
    fillMode: 'outline',
    themeColor: 'primary',
    disabled: false,
    selection: 'single',
    toggleable: true,
  },
};
