import { StoryObj, Meta, moduleMetadata } from '@storybook/angular';
import { ButtonModule } from '@progress/kendo-angular-buttons';
import { ButtonComponent } from '../../../lib/atoms/button/button.component';
import { TypographyModule } from '../../../lib/atoms/typography/typography.module';
import { IconModule } from '@components/atoms/icon/icon.module';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<ButtonComponent> = {
  title: 'Atoms/ButtonComponent',
  component: ButtonComponent,
  decorators: [
    moduleMetadata({
      imports: [ButtonModule, TypographyModule, ThemesModule, IconModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    size: {
      description: 'Specify "small" , "medium" or "large"',
      options: ['small', 'medium', 'large'],
      control: 'select',
    },
    rounded: {
      options: ['small', 'medium', 'large', 'full', 'none'],
      control: 'select',
    },
    fillMode: {
      options: ['solid', 'flat', 'outline', 'clear', 'link', 'none'],
      control: 'select',
    },
    themeColor: {
      options: [
        'primary',
        'secondary',
        'base',
        'tertiary',
        'info',
        'success',
        'warning',
        'error',
        'dark',
        'light',
        'inverse',
      ],
      control: 'select',
    },
    label: {
      control: 'text',
    },
    width: {
      control: 'text',
    },
    disabled: {
      control: 'boolean',
    },
    toggleable: {
      control: 'boolean',
    },
    iconPosition: {
      control: 'text',
    },
    onButtonClick: { action: true },
  },
  render: (args: ButtonComponent) => ({
    props: args,
    template: `<lib-button 
      [size]="'${args.size}'" 
      [rounded]="'${args.rounded}'" 
      [fillMode]="'${args.fillMode}'" 
      [themeColor]="'${args.themeColor}'" 
      [label]="'${args.label}'"
      [width]="'${args.width}'"
      [iconName]="'${args.iconName}'"
      [disabled]=${args.disabled}
      [toggleable]=${args.toggleable}
      [iconPosition]="'${args.iconPosition}'"
    >
  </lib-button>`,
  }),
};

export default meta;

type Story = StoryObj<ButtonComponent>;

export const Button: Story = {
  args: {
    size: 'large',
    rounded: 'small',
    fillMode: 'outline',
    themeColor: 'primary',
    label: 'button',
    width: '10%',
    iconName: 'home',
    disabled: false,
    toggleable: false,
    iconPosition: 'before',
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    const btn = canvas.getByRole('button');
    await userEvent.click(btn);
  },
};
