import { moduleMetadata, StoryObj, Meta } from '@storybook/angular';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { InputsModule } from '@progress/kendo-angular-inputs';
import { CheckboxComponent } from '@components/atoms/checkbox/checkbox.component';
import '@angular/localize/init';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<CheckboxComponent> = {
  title: 'Atoms/CheckboxComponent',
  component: CheckboxComponent,
  decorators: [
    moduleMetadata({
      declarations: [CheckboxComponent],
      imports: [
        InputsModule,
        TypographyModule,
        ReactiveFormsModule,
        ThemesModule,
      ],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    id: {
      description: 'Add "for" property of label for label association.',
      control: 'text',
    },
    size: {
      description: 'Specify "small" , "medium" or "large"',
      options: ['small', 'medium', 'large'],
      control: 'radio',
    },
    rounded: {
      options: ['small', 'medium', 'large', 'none'],
      control: 'radio',
    },
    labelPosition: {
      options: ['before', 'after'],
      control: 'radio',
    },
    control: {
      description: `Example: [control]="parentForm.get('form-control-name)"`,
    },
    emitValueChange: {
      description: `Event emmiter for checkbox`,
      action: true,
    },
  },
  render: (args: CheckboxComponent) => ({
    props: args,
  }),
};

export default meta;

type Story = StoryObj<CheckboxComponent>;

export const Checkbox: Story = {
  args: {
    size: 'medium',
    rounded: 'none',
    id: 'test',
    label: 'checkbox',
    labelPosition: 'before',
    control: new FormControl(),
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    const checkShow = await canvas.getByRole('checkbox');
    await userEvent.click(checkShow);
  },
};
