import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { ClipboardComponent } from '@components/atoms/clipboard/clipboard.component';
import '@angular/localize/init';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { IconModule } from '@components/atoms/icon/icon.module';
import { TooltipModule } from '@components/atoms/tooltip/tooltip.module';
import { provideAnimations } from '@angular/platform-browser/animations';
import { TypographyModule } from '@components/atoms/typography/typography.module';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<ClipboardComponent> = {
  title: 'Atoms/ClipboardComponent',
  component: ClipboardComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      declarations: [],
      imports: [ThemesModule, TooltipModule, IconModule,TypographyModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    clipboardData: {
      control: 'text',
    },
    preCopyText: {
      control: 'text',
    },
    postCopyText: {
      control: 'text',
    },
    tooltipPosition: {
      options: ['top', 'bottom', 'left', 'right'],
      control: 'radio',
    },
  },
  render: (args: ClipboardComponent) => ({
    props: args,
    template: `
    <div style="width: 100%;height: 150px;display: flex;align-items: center;justify-content: center;">
    <div style="display:flex;align-items: center">
        <lib-clipboard tooltipPosition="${args.tooltipPosition}" preCopyText="${args.preCopyText}"
          postCopyText="${args.postCopyText}" clipboardData="${args.clipboardData}"></lib-clipboard>
    </div>
  `,
  }),
};

export default meta;

type Story = StoryObj<ClipboardComponent>;

export const Clipboard: Story = {
  args: {
    clipboardData: 'Lorem ipsum',
    preCopyText: 'Copy text',
    postCopyText: 'Text copied!',
    tooltipPosition: 'top',
  },
};
