import { moduleMetadata, StoryObj, Meta } from '@storybook/angular';
import { ContainerComponent } from '@components/atoms/container/container.component';
import { ContainerModule } from '@components/atoms/container/container.module';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}
// More on default export: https://storybook.js.org/docs/angular/writing-stories/introduction#default-export
const meta: Meta<ContainerComponent> = {
  title: 'Atoms/ContainerComponent',
  component: ContainerComponent,
  decorators: [
    moduleMetadata({
      imports: [ContainerModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  // More on argTypes: https://storybook.js.org/docs/angular/api/argtypes
  argTypes: {
    c_class: {
      options: ['row', 'container', 'container-fluid'],
    },
    padding: {
      options: ['pd-small', 'pd-medium', 'pd-large'],
      control: 'select',
    },
    margin: {
      options: ['mr-small', 'mr-medium', 'mr-large'],
      control: 'select',
    },
  },
  render: (args: ContainerComponent) => ({
    props: args,
    template: `<lib-container [c_class]=c_class [padding]=padding [margin]=margin>Compliance</lib-container>`,
  }),
};
export default meta;

type Story = StoryObj<ContainerComponent>;

export const Div: Story = {
  args: {
    c_class: '',
    padding: '',
    margin: '',
  },
};
