import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { LayoutModule } from '@progress/kendo-angular-layout';
import { IconsModule } from '@progress/kendo-angular-icons';
import { ButtonsModule } from '@progress/kendo-angular-buttons';
import { InputsModule } from '@progress/kendo-angular-inputs';
import { LabelModule } from '@progress/kendo-angular-label';
import { DropDownsModule } from '@progress/kendo-angular-dropdowns';
import { ExpansionPanelComponent } from '@components/atoms/expansion-panel/expansion-panel.component';
import { TypographyComponent } from '@components/atoms/typography/typography.component';
import { ContainerModule } from '@components/atoms/container/container.module';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<ExpansionPanelComponent> = {
  title: 'Atoms/ExpansionPanelComponent',
  component: ExpansionPanelComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      declarations: [TypographyComponent],
      imports: [
        LayoutModule,
        IconsModule,
        ButtonsModule,
        InputsModule,
        LabelModule,
        DropDownsModule,
        ContainerModule,
        ThemesModule,
      ],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    title: {
      control: 'text',
    },
    subtitle: {
      control: 'text',
    },
    data: {
      control: 'text',
    },
    customHeader: {
      description: `Removes title and subtile fields and we can pass our own custom HTML inside <header></header> tag.`,
      table: {
        defaultValue: {
          summary: false,
        },
      },
    },
    disabled: {
      control: 'boolean',
    },
  },
  render: (args: ExpansionPanelComponent) => ({
    props: args,
    template: `
      <lib-expansion-panel [title]="title" [subtitle]="subtitle" [expanded]="expanded" [customHeader]="customHeader" [disabled]="disabled">
        <header>
          <span>Custom Header</span>
        </header>
        <lib-typography
          [defaultText]="data"
          type="P_MULISH_14_REGULAR"
        >
        </lib-typography>
      </lib-expansion-panel>
    `,
  }),
};

export default meta;

type Story = StoryObj<ExpansionPanelComponent>;

export const Expansion: Story = {
  args: {
    title: 'title',
    subtitle: 'subtitle',
    data: 'expansion panel content',
    expanded: true,
    customHeader: false,
    disabled: false,
  },
};
