import { moduleMetadata, StoryObj, Meta } from '@storybook/angular';
import { IconModule } from '@components/atoms/icon/icon.module';
import { IconComponent } from '@components/atoms/icon/icon.component';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<IconComponent> = {
  title: 'Atoms/IconComponent',
  component: IconComponent,
  decorators: [
    moduleMetadata({
      imports: [IconModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    name: {
      control: 'select',
      options: [
        'scan',
        'data',
        'tasks',
        'information',
        'search',
        'home',
        'close',
        'launch',
        'arrow-right',
        'menu',
        'idea',
        'chevron-up',
        'chevron-right',
        'chevron-down',
        'chevron-left',
        'circle-checked',
        'delete',
        'caret-down',
        'upload',
        'edit-filter',
        'overflow-menu-vertical',
        'checkbox',
        'data-diagram',
        'error-filled',
        'warning-filled',
        'notification-filled',
        'cross-filled',
        'filter',
        'reset',
        'GCP',
        'Azure',
        'aws',
        'kubernates',
        'docker',
        'add',
        'copy',
        'eye-closed',
        'eye-open',
        'edit',
        'logout',
      ],
    },
    fontSize: {
      control: 'number',
    },
    type: {
      control: 'select',
      options: ['success', 'error'],
      description:
        'This is an optional field. It is used to provide theme color to icons.',
    },
  },
  render: (args: IconComponent) => ({
    props: args,
  }),
};
export default meta;

type Story = StoryObj<IconComponent>;

export const Icon: Story = {
  args: {
    name: 'home',
    fontSize: 20,
  },
};
