import { moduleMetadata, StoryObj, Meta } from '@storybook/angular';
import { LabelModule } from '@progress/kendo-angular-label';
import { LabelComponent } from '@components/atoms/label/label.component';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import '@angular/localize/init';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { TextboxModule } from '@components/atoms/textbox/textbox.module';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

// More on default export: https://storybook.js.org/docs/angular/writing-stories/introduction#default-export
const meta: Meta<LabelComponent> = {
  title: 'Atoms/LabelComponent',
  component: LabelComponent,
  decorators: [
    moduleMetadata({
      imports: [LabelModule, TypographyModule, ThemesModule, TextboxModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    text: {
      control: 'text',
    },
  },
  render: (args: LabelComponent) => ({
    props: args,
    template: `<lib-label [text]="'${args.text}'" [hiddenLabel]="'${args.text}'" [labelFont]="'${args.labelFont}'">
      <lib-textbox placeholder="text" fillMode="solid">
      </lib-textbox>
    </lib-label>`,
  }),
};
export default meta;

type Story = StoryObj<LabelComponent>;

export const Label: Story = {
  args: {
    text: 'label',
  },
};
