import { moduleMetadata, StoryObj, Meta } from '@storybook/angular';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { LoadingSpinnerComponent } from '@components/atoms/loading-spinner/loading-spinner.component';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<LoadingSpinnerComponent> = {
  title: 'Atoms/LoadingSpinnerComponent',
  component: LoadingSpinnerComponent,
  decorators: [
    moduleMetadata({
      imports: [ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    width: {
      control: 'number',
    },
    height: {
      control: 'number',
    },
    visible: {
      control: 'boolean',
    },
  },
  render: (args: LoadingSpinnerComponent) => ({
    props: args,
    template: `
      <lib-loading-spinner [width]=${args.width} [height]=${args.height} [visible]=${args.visible}></lib-loading-spinner>
      `,
  }),
};

export default meta;

type Story = StoryObj<LoadingSpinnerComponent>;

export const LoadingSpinner: Story = {
  args: {
    width: 100,
    height: 100,
    visible: true,
  },
};
