import '@angular/localize/init';
import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { NotificationComponent } from '@components/atoms/notification/notification.component';
import { NotificationModule } from '@components/atoms/notification/notification.module';
import { ButtonsModule } from '@progress/kendo-angular-buttons';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { APP_INITIALIZER } from '@angular/core';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

enum Type {
  success = 'success',
  error = 'error',
  warning = 'warning',
  info = 'info',
  none = 'none',
}
const meta: Meta<NotificationComponent> = {
  title: 'Atoms/NotificationComponent',
  component: NotificationComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      declarations: [],
      imports: [NotificationModule, ButtonsModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    type: {
      options: Type,
      control: 'radio',
    },
    timeout: {
      control: 'number',
    },
  },
  render: (args: NotificationComponent) => ({
    props: args,
    template: `
    <lib-notification [type]="type" [timeout]="${args.timeout}">Hello</lib-notification>`,
  }),
};

export default meta;

type Story = StoryObj<NotificationComponent>;

export const notification: Story = {
  args: {
    type: Type.success,
    timeout: 2,
  },
};
