import { moduleMetadata, StoryObj, Meta } from '@storybook/angular';
import { InputsModule } from '@progress/kendo-angular-inputs';
import { RadioButtonComponent } from '@components/atoms/radio-button/radio-button.component';
import { FormControl, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<RadioButtonComponent> = {
  title: 'Atoms/Radio Button Component',
  component: RadioButtonComponent,
  decorators: [
    moduleMetadata({
      imports: [
        InputsModule,
        ReactiveFormsModule,
        TypographyModule,
        ThemesModule,
        FormsModule,
      ],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    size: {
      description: 'Represents the possible size options of the inputs.',
      options: ['small', 'medium', 'large', 'none'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'medium',
        },
      },
    },
    name: {
      description: `Name attribute.`,
      control: 'text',
      type: {
        required: true,
        name: 'string',
      },
    },
    value: {
      description: `Value to be assigned to the field.`,
      control: 'text',
      type: {
        required: true,
        name: 'string',
      },
    },
    label: {
      description: `Text to be shown in the label. If this is empty string, no label will be shown.`,
      control: 'text',
      type: {
        required: true,
        name: 'string',
      },
    },
    labelPosition: {
      description: `Position of the label. Default value is 'before'.`,
      options: ['before', 'after'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'before',
        },
      },
    },
    id: {
      description: `Id to be assigned to the input element.`,
      control: 'text',
      type: {
        required: true,
        name: 'string',
      },
    },
    model: {
      description: `Value to be sent inside [(ngModel)] for template driven forms, don't use it with control field. Example1: [(model)]="model", Example2: (model)="model" (modelChange)="modelChange($event)"`,
    },
    control: {
      description: `FormControl to be used in reactive forms, don't use it with model field. Example: [control]="parentForm.get('form-control-name)"`,
    },
    emitModelChange: {
      action: true,
    },
  },
  render: (args: RadioButtonComponent) => ({
    props: args,
    template: `
      <lib-radio-button [size]="size" [name]="name" [value]="value" [label]="label" [labelPosition]="labelPosition" [id]="id" [(model)]="model" [control]="control"></lib-radio-button>
      &nbsp;
      <lib-radio-button [size]="size" [name]="name" [value]="'female'" [label]="'Female'" [labelPosition]="labelPosition" [id]="female" [(model)]="model" [control]="control"></lib-radio-button>
    `,
  }),
};

export default meta;

type Story = StoryObj<RadioButtonComponent>;

export const RadioButton: Story = {
  args: {
    size: 'medium',
    name: 'gender',
    value: 'male',
    label: 'Male',
    labelPosition: 'before',
    id: 'male',
    model: 'Not available',
    control: new FormControl(),
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    await userEvent.click(canvas.getByLabelText('Male'));
  },
};
