import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { SearchBoxComponent } from '@components/atoms/search-box/search-box.component';
import { AutoCompleteModule } from '@progress/kendo-angular-dropdowns';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { IconModule } from '@components/atoms/icon/icon.module';
import { InputsModule } from '@progress/kendo-angular-inputs';
import { userEvent, within } from '@storybook/test';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<SearchBoxComponent> = {
  title: 'Atoms/SearchBoxComponent',
  component: SearchBoxComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      imports: [
        AutoCompleteModule,
        ThemesModule,
        IconModule,
        InputsModule,
        ReactiveFormsModule,
      ],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    filterable: {
      options: [true, false],
      control: 'radio',
    },
    size: {
      options: ['small', 'medium', 'large', 'none'],
      control: 'radio',
    },
    icon: {
      control: 'text',
    },
    autoComplete: {
      options: [true, false],
      control: 'radio',
    },
    rounded: {
      options: ['small', 'medium', 'large', 'full', 'none'],
      control: 'radio',
    },
    fillMode: {
      options: ['solid', 'flat', 'outline', 'none'],
      control: 'radio',
    },
    clearButton: {
      description: 'Specify "true" or "false"',
      options: [true, false],
      control: 'radio',
    },
    placeholder: {
      control: 'text',
    },
    width: {
      control: 'number',
    },
    suggest: {
      options: [true, false],
      control: 'radio',
    },
    emitValueChange: {
      action: true,
    },
    value: {
      control: 'text',
    },
  },
  render: (args: SearchBoxComponent) => ({
    props: args,
  }),
};
export default meta;

type Story = StoryObj<SearchBoxComponent>;

export const SearchBox: Story = {
  args: {
    source: ['Albania', 'Andorra', 'Armenia', 'Austria', 'Azerbaijan'],
    filterable: true,
    clearButton: true,
    rounded: 'medium',
    fillMode: 'solid',
    icon: 'search',
    autoComplete: false,
    placeholder: 'Search',
    value: 'Search Term',
    suggest: true,
  },
};

export const Textbox: Story = {
  args: {
    source: ['Albania', 'Andorra', 'Armenia', 'Austria', 'Azerbaijan'],
    icon: 'search',
    autoComplete: false,
    placeholder: 'Search',
    width: 200,
    value: '',
    control: new FormControl(),
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    await userEvent.click(canvas.getByRole('textbox'));
  },
};

export const autocomplete: Story = {
  args: {
    source: ['Albania', 'Andorra', 'Armenia', 'Austria', 'Azerbaijan'],
    filterable: true,
    autoComplete: true,
    placeholder: 'Search',
    suggest: true,
    value: '',
    control: new FormControl(),
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    await userEvent.click(canvas.getByRole('combobox'));
  },
};
