import { moduleMetadata, StoryObj, Meta } from '@storybook/angular';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { SliderComponent } from '@components/atoms/slider/slider.component';
import { TypographyModule } from '@components/atoms/typography/typography.module';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<SliderComponent> = {
  title: 'Atoms/SliderComponent',
  component: SliderComponent,
  decorators: [
    moduleMetadata({
      imports: [TypographyModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    title: {
      options: ['Enter your title'],
    },
    frameNumber: {
      control: 'number',
    },
  },
  render: (args: SliderComponent) => ({
    props: args,
    template: `
      <lib-slider [opened]="opened" [title]="title" [frameNumber]="frameNumber"></lib-slider>
      `,
  }),
};
export default meta;

type Story = StoryObj<SliderComponent>;

export const SliderPanel: Story = {
  args: {
    opened: true,
    title: 'Enter your title',
    frameNumber: 1,
  },
};
