import { moduleMetadata, StoryObj, Meta } from '@storybook/angular';
import { InputsModule } from '@progress/kendo-angular-inputs';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import '@angular/localize/init';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';
import { TextAreaComponent } from '@components/atoms/text-area/text-area.component';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<TextAreaComponent> = {
  title: 'Atoms/TextAreaComponent',
  component: TextAreaComponent,
  decorators: [
    moduleMetadata({
      imports: [InputsModule, ReactiveFormsModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    placeholder: {
      control: 'text',
    },
    resizable: {
      description: 'Specify "both" , "vertical" or "horizontal"',
      options: ['vertical', 'horizontal', 'both'],
      control: 'radio',
    },
    rows: {
      description: 'No of rows we can add, example 10 or 20',
      control: 'number',
    },
    cols: {
      description: 'No of cols we can add, example 50 or 80',
      control: 'number',
    },
    width: {
      description:
        'we can set the width of the text area, example auto or 100, 200 or 300',
      control: 'text',
    },
    emitValueChange: {
      action: true,
    },
    value: {
      control: 'text',
    },
    control: {
      description: `Example: [control]="parentForm.get('form-control-name')"`,
    },
  },
  render: (args: TextAreaComponent) => ({
    props: args,
  }),
};

export default meta;

type Story = StoryObj<TextAreaComponent>;

export const TextArea: Story = {
  args: {
    placeholder: 'Enter here',
    resizable: 'both',
    cols: 50,
    rows: 5,
    width: '500',
    control: new FormControl(),
    value: '',
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    await userEvent.click(canvas.getByRole('textbox'));
  },
};
