import { moduleMetadata, StoryObj, Meta } from '@storybook/angular';
import { InputsModule } from '@progress/kendo-angular-inputs';
import { TextboxComponent } from '@components/atoms/textbox/textbox.component';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import '@angular/localize/init';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';
import { IconModule } from '@components/atoms/icon/icon.module';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<TextboxComponent> = {
  title: 'Atoms/TextboxComponent',
  component: TextboxComponent,
  decorators: [
    moduleMetadata({
      imports: [InputsModule, ReactiveFormsModule, ThemesModule, IconModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    size: {
      description: 'Specify "small" , "medium" or "large"',
      options: ['small', 'medium', 'large'],
      control: 'radio',
    },
    rounded: {
      options: ['small', 'medium', 'large', 'full', 'none'],
      control: 'radio',
    },
    fillMode: {
      options: ['solid', 'flat', 'outline', 'none'],
      control: 'radio',
    },
    clearButton: {
      description: 'Specify "true" or "false"',
      options: [true, false],
      control: 'radio',
    },
    isDisabled: {
      options: [true, false],
      control: 'radio',
    },
    placeholder: {
      control: 'text',
    },
    width: {
      control: 'text',
    },
    readonly: {
      options: [true, false],
      control: 'radio',
    },
    control: {
      description: `Example: [control]="parentForm.get('form-control-name')"`,
    },
    emitValueChange: {
      action: true,
    },
    value: {
      control: 'text',
    },
    type: {
      description:
        'we can use type option to manage the password type field, we need to pass type `password`.',
      control: 'text',
    },
    showEyeIcon: {
      options: [true, false],
      control: 'radio',
    },
    onFocus: {
      action: true,
    },
    onBlur: {
      action: true,
    }
  },
  render: (args: TextboxComponent) => ({
    props: args,
  }),
};

export default meta;

type Story = StoryObj<TextboxComponent>;

export const Textbox: Story = {
  args: {
    clearButton: false,
    isDisabled: false,
    placeholder: 'textbox',
    width: '300',
    readonly: false,
    size: 'medium',
    rounded: 'medium',
    fillMode: 'solid',
    control: new FormControl(),
    value: '',
    type: '',
    showEyeIcon: false,
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    await userEvent.click(canvas.getByRole('textbox'));
  },
};
