import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { DateInputsModule } from '@progress/kendo-angular-dateinputs';
import { TimeschedulerComponent } from '@components/atoms/timescheduler/timescheduler.component';
import { TypographyComponent } from '@components/atoms/typography/typography.component';
import '@angular/localize/init';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { APP_INITIALIZER } from '@angular/core';
import { userEvent, within } from '@storybook/test';
import { FormControl, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<TimeschedulerComponent> = {
  title: 'Atoms/TimeschedulerComponent',
  component: TimeschedulerComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      declarations: [TypographyComponent],
      imports: [
        DateInputsModule,
        FormsModule,
        ReactiveFormsModule,
        ThemesModule,
      ],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    size: {
      options: ['small', 'medium', 'large'],
      control: 'radio',
    },
    rounded: {
      options: ['small', 'medium', 'large', 'full', 'none'],
      control: 'radio',
    },
    fillMode: {
      options: ['solid', 'flat', 'outline', 'clear', 'link', 'none'],
      control: 'radio',
    },
    placeholder: {
      control: 'text',
    },
    model: {
      description: `Value to be sent in template driven forms. Example1: [(model)]="model", Example2: (model)="model" (modelChange)="modelChange($event)"`,
    },
    emitModelChange: {
      action: true,
    },
  },
  render: (args: TimeschedulerComponent) => ({
    props: args,
    template: `
    <!-- Use the variable in model attribute to read the selected Date/Time  -->
    <lib-timescheduler
      [format]="format"
      [size]="size"
      [rounded]="rounded"
      [fillMode]="fillMode"
      [isDisabled]="isDisabled"
      [readonly]="readonly"
      [placeholder]="placeholder"
      [(model)]="model"
    >
    </lib-timescheduler>
    `,
  }),
};

export default meta;

type Story = StoryObj<TimeschedulerComponent>;

export const TimeScheduler: Story = {
  args: {
    size: 'large',
    rounded: 'small',
    fillMode: 'outline',
    isDisabled: false,
    readonly: false,
    placeholder: 'Select Date & Time',
    format: 'MM/dd/yyyy hh:mm a',
    model: new Date('2014-05-22'),
    control: new FormControl(),
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    const btn = canvas.getByRole('button');
    await userEvent.click(btn);
  },
};
