import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { TooltipsModule } from '@progress/kendo-angular-tooltip';
import { TooltipComponent } from '@components/atoms/tooltip/tooltip.component';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import '@angular/localize/init';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<TooltipComponent> = {
  title: 'Atoms/TooltipComponent',
  component: TooltipComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      imports: [TooltipsModule, TypographyModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    position: {
      options: ['top', 'bottom', 'left', 'right'],
      control: 'radio',
    },
    showOn: {
      options: ['hover', 'click', 'none'],
      control: 'radio',
    },
    closable: {
      options: [true, false],
      control: 'radio',
    },
    title: {
      control: 'text',
    },
    tooltipWidth: {
      control: 'number',
      description: 'optional. For custom tooltip Width.',
    },
    tooltipHeight: {
      control: 'number',
      description: 'optional. For custom tooltip Height.',
    },
    callout: {
      options: [true, false],
      control: 'radio',
    },
  },
  render: (args: TooltipComponent) => ({
    props: args,
    template: `
    <!-- the outer divs are for storybook alignments. Capture the code from <lib-tooltip>
     Wrap tooltip content inside div with class="tooltip-content" -->
    <div style="width: 100%;height: 150px;display: flex;align-items: center;justify-content: center;">
    <div style="width: 109px;">
      <lib-tooltip
          position=${args.position} 
          showOn=${args.showOn} 
          title=${args.title}
          [callout]=${args.callout}
          [closable]=${args.closable}
        >
        <p>test</p>
        <div class="tooltip-content" style="padding: 20px 24px;">
          <h4>How to get Access key</h4><br />
          <ol>
            <li>Login into your AWS Account.</li>
            <li>Open the IAM console at https://console.aws.amazon.com/iam/.</li>
            <li>On the navigation menu, choose Users.</li>
          </ol>
        </div>
      </lib-tooltip>
    </div>`,
  }),
};

export default meta;

type Story = StoryObj<TooltipComponent>;

export const Tooltip: Story = {
  args: {
    position: 'right',
    showOn: 'hover',
    title: 'Sample Tooltip',
    closable: false,
    callout: true,
    tooltipWidth: 90,
    tooltipHeight: 30,
  },
};
