import { TypographyComponent } from '@components/atoms/typography/typography.component';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { moduleMetadata, StoryObj, Meta } from '@storybook/angular';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

// More on default export: https://storybook.js.org/docs/angular/writing-stories/introduction#default-export
const meta: Meta<TypographyComponent> = {
  title: 'Atoms/TypographyComponent',
  component: TypographyComponent,
  decorators: [
    moduleMetadata({
      imports: [ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  // More on argTypes: https://storybook.js.org/docs/angular/api/argtypes
  argTypes: {
    type: {
      description: 'Set the font style.',
      options: [
        // h1 styles
        'H1_MULISH_36_BOLD',
        'H1_MULISH_36_REGULAR',
        'H1_MULISH_36_LIGHT',
        // h2 styles
        'H2_MULISH_24_BOLD',
        'H2_MULISH_24_REGULAR',
        'H2_MULISH_24_LIGHT',
        // h3 styles
        'H3_MULISH_18_BOLD',
        'H3_MULISH_18_REGULAR',
        'H3_MULISH_18_LIGHT',
        // h4, h5 and h6 styles
        'H4_MULISH_14_BOLD',
        'H5_MULISH_14_REGULAR',
        'H6_MULISH_14_LIGHT',
        // p 16px styles
        'P_MULISH_16_BOLD',
        'P_MULISH_16_REGULAR',
        'P_MULISH_16_REGULAR_ITALIC',
        'P_MULISH_16_LIGHT',
        'P_MULISH_16_LIGHT_ITALIC',
        // p 14px styles
        'P_MULISH_14_BOLD',
        'P_MULISH_14_REGULAR',
        'P_MULISH_14_REGULAR_ITALIC',
        'P_MULISH_14_LIGHT',
        'P_MULISH_14_LIGHT_ITALIC',
        // small 12px styles
        'SMALL_MULISH_12_BOLD',
        'SMALL_MULISH_12_REGULAR',
        'SMALL_MULISH_12_REGULAR_ITALIC',
        'SMALL_MULISH_12_LIGHT',
        'SMALL_MULISH_12_LIGHT_ITALIC',
        // Custom
        'BASE_TEXT_MULISH',
      ],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'P_MULISH_14_REGULAR',
        },
      },
    },
    defaultText: {
      description:
        'Accepts the text on which the selected font should be applied.',
      table: {
        defaultValue: {
          summary: 'defaultText',
        },
      },
    },
  },
  render: (args: TypographyComponent) => ({
    props: args,
    template: `<lib-typography 
    type="${args.type}"
    defaultText="Default Sample Text"
    >Sample Text
    </lib-typography>`,
  }),
};

export default meta;

type Story = StoryObj<TypographyComponent>;

// More on component templates: https://storybook.js.org/docs/angular/writing-stories/introduction#using-args

export const type: Story = {
  args: {},
};
