import { ButtonComponent } from '@components/atoms/button/button.component';
import { CardActionsComponent } from '@components/molecules/card/card-actions/card-actions.component';
import { CardBodyComponent } from '@components/molecules/card/card-body/card-body.component';
import { CardFooterComponent } from '@components/molecules/card/card-footer/card-footer.component';
import { CardHeaderComponent } from '@components/molecules/card/card-header/card-header.component';
import { CardComponent } from '@components/molecules/card/card.component';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { ButtonModule } from '@progress/kendo-angular-buttons';

import { LayoutModule } from '@progress/kendo-angular-layout';
import { moduleMetadata, Meta, StoryObj } from '@storybook/angular';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<CardComponent> = {
  title: 'Molecules/Card Component',
  component: CardComponent,
  decorators: [
    moduleMetadata({
      declarations: [
        CardComponent,
        CardHeaderComponent,
        CardFooterComponent,
        CardActionsComponent,
        CardBodyComponent,
        ButtonComponent,
      ],
      imports: [LayoutModule, TypographyModule, ButtonModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    orientation: {
      options: ['vertical', 'horizontal'],
      control: 'radio',
    },
    width: {
      control: 'text',
    },
  },
  render: (args: CardComponent) => ({
    props: args,
    template: `
      <lib-card width=${args.width} orientation=${args.orientation}>
        <lib-card-header cardNumber=1 title="Card Title" subtitle="Card Subtitle">
        </lib-card-header>
        <lib-card-body>
          <lib-typography
            defaultText="The Black Sea is bordered by Ukraine, Romania, Bulgaria, Turkey,
            Georgia, and Russia. It has a positive water balance; that is, a net
            outflow of water 300 km3 per year."
            type="H5_MULISH_14_REGULAR">
          </lib-typography>
        </lib-card-body>
        <lib-card-actions orientation="horizontal" layout="center">
          <lib-button themeColor="primary" fillMode="clear" label="Explore"></lib-button>
          <lib-button themeColor="primary" fillMode="clear" label="Share"></lib-button>
        </lib-card-actions>
        <lib-card-footer>
          <lib-typography defaultText="Created By @alex" type="H5_MULISH_14_REGULAR"></lib-typography>
        </lib-card-footer>
      </lib-card>
    `,
  }),
};

export default meta;
type Story = StoryObj<CardComponent>;

export const Card: Story = {
  args: {
    orientation: 'vertical',
    width: '400px'
  },
};
