import '@angular/localize/init';
import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { ButtonsModule } from '@progress/kendo-angular-buttons';
import { DialogComponent } from '@components/molecules/dialog/dialog.component';
import { DialogsModule } from '@progress/kendo-angular-dialog';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { DialogActionsComponent } from '@components/molecules/dialog/dialog-actions/dialog-actions.component';
import { DialogTitlebarComponent } from '@components/molecules/dialog/dialog-titlebar/dialog-titlebar.component';
import { ButtonComponent } from '@components/atoms/button/button.component';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<DialogComponent> = {
  title: 'Molecules/Dialog Component',
  component: DialogComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      declarations: [
        DialogActionsComponent,
        DialogTitlebarComponent,
        ButtonComponent,
        DialogComponent,
      ],
      imports: [ButtonsModule, DialogsModule, TypographyModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    width: {
      control: 'text',
    },
    height: {
      control: 'text',
    },
    minWidth: {
      control: 'number',
    },
    maxWidth: {
      control: 'number',
    },
    opened: {
      control: 'boolean',
    },
    minHeight: {
      control: 'number',
    },
    maxHeight: {
      control: 'number',
    },
  },
  render: (args: DialogComponent) => ({
    props: args,
    template: `
        <lib-dialog 
        *ngIf="opened"
        [width]='width' 
        [height]='height' 
        [minWidth]='minWidth' 
        [maxWidth]='maxWidth' 
        [minHeight]='minHeight' 
        [maxHeight]='maxHeight'>
          <lib-dialog-titlebar title="Title" (closeDialog)="opened=false;" [hideCloseIcon]="false"></lib-dialog-titlebar>
          <p>Hello</p>
          <lib-dialog-actions layout="normal">
            <lib-button themeColor="primary" label="Okay" (click)="opened=false"></lib-button>
          </lib-dialog-actions>
        </lib-dialog>
      `,
  }),
};

export default meta;

type Story = StoryObj<DialogComponent>;

export const Dialog: Story = {
  args: {
    opened: true,
    width: '50%',
    height: '50%',
    minWidth: 200,
    maxWidth: 500,
    minHeight: 200,
    maxHeight: 1200,
  },
};

export const DialogInteraction: Story = {
  args: {
    opened: true,
    width: '50%',
    height: '50%',
    minWidth: 200,
    maxWidth: 1200,
    minHeight: 200,
    maxHeight: 1200,
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    const closeButton = canvas.getByTitle('Close');
    await userEvent.click(closeButton);
  },
};
