import '@angular/localize/init';
import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { ButtonsModule } from '@progress/kendo-angular-buttons';
import { GridFormComponent } from '@components/molecules/grid/grid-form/grid-form.component';
import { GridModule } from '@progress/kendo-angular-grid';
import { InputsModule } from '@progress/kendo-angular-inputs';
import { DialogModule } from '@progress/kendo-angular-dialog';
import { FormControl, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { GridsModule } from '@components/molecules/grid/grid.module';
import { DropdownModule } from '@components/atoms/dropdown/dropdown.module';
import { LabelsModule } from '@components/atoms/label/label.module';
import { IconModule } from '@components/atoms/icon/icon.module';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<GridFormComponent> = {
  title: 'Molecules/GridFormComponent',
  component: GridFormComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      declarations: [],
      imports: [
        ButtonsModule,
        ThemesModule,
        GridModule,
        IconModule,
        InputsModule,
        LabelsModule,
        DialogModule,
        FormsModule,
        ReactiveFormsModule,
        TypographyModule,
        GridsModule,
        DropdownModule,
      ],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    gridData: {
      control: 'object',
    },
    columnData: {
      control: 'object',
    },
    controls: {
      control: 'object',
    },
  },
  render: (args: GridFormComponent) => ({
    props: args,
    template: `
    <lib-grid-form [gridData]="gridData" [columnData]="columnData" [controls]="controls" [showToolbar]="true">
      <header>
         <lib-label text="Select Specific instances to run the scan" labelFont="H4_Mulish_14_Bold" class="grid-header"></lib-label>
      </header>
      <div class="header-add-btn">
        <lib-typography id="btn-text" defaultText="Add New Item" type="P_MULISH_16_REGULAR"></lib-typography>
      </div>
      <ng-template #tmp1 let-dataItem let-formGroup="formGroup">
        <lib-dropdown [data]="[{id:'1', name:'cred1'}, {id:'2', name:'cred2'}]" [model]="dataItem.CredentialId" textField='name' valueField='id' [valuePrimitive]='true' placeholder="Select Credential"
        [control]="formGroup.get('CredentialId')"
        ></lib-dropdown>
      </ng-template>
      <ng-template #tmp2 let-dataItem>
       {{ dataItem.CredentialId }} 
      </ng-template>
    </lib-grid-form>
    `,
  }),
};

export default meta;

type Story = StoryObj<GridFormComponent>;

export const Grid: Story = {
  args: {
    gridData: [
      {
        id: 1,
        ProductID: 1,
        ProductName: 'Chai',
        UnitPrice: 18,
        CategoryID: 1,
        CategoryName: 'Beverages',
        CredentialId: '1',
      },
      {
        id: 2,
        ProductID: 2,
        ProductName: 'Chang',
        UnitPrice: 19,
        CategoryID: 1,
        CategoryName: 'Beverages',
        CredentialId: '2',
      },
      {
        id: 3,
        ProductID: 3,
        ProductName: 'Aniseed Syrup',
        UnitPrice: 10,
        CategoryID: 2,
        CategoryName: 'Condiments',
        CredentialId: '',
      },
    ],
    columnData: [
      {
        field: 'ProductName',
        title: 'Product Name',
      },
      {
        field: 'UnitPrice',
        title: 'Unit Price',
        editor: 'numeric',
        type: 'number',
      },
      {
        field: 'CategoryID',
        title: 'CategoryID',
      },
      {
        field: 'CategoryName',
        title: 'CategoryName',
      },
      {
        field: 'CredentialId',
        title: 'Credential',
        custom: true,
      },
    ],
    controls: {
      ProductName: '',
      UnitPrice: 0,
      CategoryID: '0',
      CategoryName: '',
      CredentialId: '1',
    },
  },
};
