import { moduleMetadata, Meta, StoryObj } from '@storybook/angular';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { GridLayoutComponent } from '@components/molecules/grid-layout/grid-layout.component';
import { GridLayoutModule } from '@components/molecules/grid-layout/grid-layout.module';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<GridLayoutModule> = {
  title: 'Molecules/Grid Layout Component',
  component: GridLayoutComponent,
  decorators: [
    moduleMetadata({
      imports: [ThemesModule, GridLayoutModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    horizontalAlign: {
      description:
        'Represents the possible horizontal alignment options of the GridLayout.',
      options: ['start', 'center', 'end', 'stretch'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'stretch',
        },
      },
    },
    verticalAlign: {
      description:
        'Represents the possible vertical alignment options of the GridLayout.',
      options: ['top', 'middle', 'bottom', 'stretch'],
      control: 'radio',
      table: {
        defaultValue: {
          summary: 'stretch',
        },
      },
    },
    rowGap: {
      description:
        'Defines the row gap between the elements. The size of the gap can be passed as a number, which will be treated as pixels, or as a string, which allows setting custom CSS units.',
      table: {
        defaultValue: {
          summary: '0',
        },
      },
    },
    columnGap: {
      description:
        'Defines the column gap between the elements. The size of the gap can be passed as a number, which will be treated as pixels, or as a string, which allows setting custom CSS units.',
      table: {
        defaultValue: {
          summary: '0',
        },
      },
    },
    rows: {
      description: `Specifies the number of rows and their height. The number of elements in the array defines the number of rows. Each array element defines the size of the corresponding row. The possible array values are:
      1)number - Defines the size in pixels.
      2)string - Enables the usage of arbitrary units e.g. 20% or auto.
      3)GridLayoutRowSize - Configuration object, which accepts a height key.`,
    },
    columns: {
      description: `Specifies the number of columns and their widths. The number of elements in the array defines the number of columns. Each array element defines the size of the corresponding column. The possible array values are:
      1)number - Defines the size in pixels.
      2)string - Enables the usage of arbitrary units e.g. 20% or auto.
      3)GridLayoutColSize - Configuration object, which accepts a width key.`,
    },
    rowNumber: {
      description: `Property of lib-grid-layout-item: Sets the row of the item in the GridLayout.`,
    },
    columnNumber: {
      description: `Property of lib-grid-layout-item: Sets the column of the item in the GridLayout.`,
    },
    rowSpan: {
      description:
        'Property of lib-grid-layout-item: Determines how many rows will the item span. The default size is one row.',
      table: {
        defaultValue: {
          summary: '1',
        },
      },
    },
    colSpan: {
      description:
        'Property of lib-grid-layout-item: Determines how many columns will the item span. The default size is one column.',
      table: {
        defaultValue: {
          summary: '1',
        },
      },
    },
  },
  render: (args) => ({
    props: args,
    template: `
      <lib-grid-layout [horizontalAlign]="horizontalAlign" [verticalAlign]="verticalAlign" [rowGap]="rowGap" [columnGap]="columnGap" [rows]="rows" [columns]="columns">
        <lib-grid-layout-item [rowNumber]="rowNumber" [columnNumber]="columnNumber" [rowSpan]="rowSpan" [colSpan]="colSpan">
          Item 1
        </lib-grid-layout-item>
        <lib-grid-layout-item >
          Item 2
        </lib-grid-layout-item>
        <lib-grid-layout-item >
          Item 3
        </lib-grid-layout-item>
        <lib-grid-layout-item >
          Item 4
        </lib-grid-layout-item>
      </lib-grid-layout>  
    `,
  }),
};

export default meta;

type Story = StoryObj<GridLayoutModule>;

export const GridLayout: Story = {
  args: {
    horizontalAlign: 'stretch',
    verticalAlign: 'stretch',
    rowGap: 0,
    columnGap: 0,
    rows: [90, 90, 90],
    columns: [90, 90, 90],
    rowNumber: 1,
    columnNumber: 1,
    rowSpan: 2,
    colSpan: 2,
  },
};
