import '@angular/localize/init';
import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { ButtonsModule } from '@progress/kendo-angular-buttons';
import { LibGridComponent } from '@components/molecules/grid/grid.component';
import { GridModule } from '@progress/kendo-angular-grid';
import { InputsModule } from '@progress/kendo-angular-inputs';
import { DialogModule } from '@progress/kendo-angular-dialog';
import { FormsModule } from '@angular/forms';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { GridsModule } from '@components/molecules/grid/grid.module';
import { ButtonModule } from '@components/atoms/button/button.module';
import { provideAnimations } from '@angular/platform-browser/animations';
import { moreVerticalIcon } from '@progress/kendo-svg-icons';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<LibGridComponent> = {
  title: 'Molecules/GridComponent',
  component: LibGridComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      declarations: [],
      imports: [
        ButtonModule,
        ButtonsModule,
        ThemesModule,
        GridModule,
        InputsModule,
        DialogModule,
        FormsModule,
        TypographyModule,
        GridsModule,
      ],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    gridData: {
      control: 'object',
    },
    columnData: {
      control: 'object',
    },
    pageSize: {
      control: 'number',
    },
    pageable: {
      control: 'boolean',
    },
    sortable: {
      control: 'boolean',
    },
    groupable: {
      control: 'boolean',
    },
    reorderable: {
      control: 'boolean',
    },
    resizable: {
      control: 'boolean',
    },
    checkBoxWidth: {
      control: 'number',
    },
    checkBoxColumnMenu: {
      control: 'boolean',
    },
    checkBoxColumnShowSelectedAll: {
      control: 'boolean',
    },
    checkBoxRequired: {
      control: 'boolean',
    },
    height: {
      control: 'number',
    },
    gridRowSelectByValue: {
      control: 'text',
    },
    navigable: {
      control: 'boolean',
    },
    mode: {
      description: 'Specify "single" , "multiple"',
      options: ['single', 'multiple'],
      control: 'select',
    },
    selectedRowData: {
      action: true,
    },
    noRecordsContent: {
      control: 'text',
    },
    showToolbar: {
      control: 'boolean',
    },
    kebabList: {
      control: 'object',
    },
    icon: {
      control: 'text',
    },
    showKebab: {
      control: 'boolean',
    },
    preSelected: {
      description:
        'Specifies the values(like rowId) to mark the specific rows as checked',
    },
    ignoreColumnCellClick: {
      control: 'object',
      description: 'Specifies the column numbers to not allow the clickability',
    },
    allowColumnClick: {
      control: 'boolean',
    },
    customExpandRowColumn:{
      control:'boolean',
      description:"Specifies whether to show the expand row column as last column of grid"
    },
    expandRowIndex:{
      control: 'object',
      description: 'Specifies the row indices(start from 0) to expand by default ',
    },
    deleteColumnWidth:{
      control:'number'
    },
    expandRowColumnWidth:{
      control:'number'
    },
    onDelete:{
      action:true
    }
  },
  render: (args: LibGridComponent) => ({
    props: args,
    template: `
      <lib-grid 
      [gridData]="gridData" 
      [columnData]="columnData"  
      [title]="title" 
      [pageSize]="'${args.pageSize}'" 
      [pageable]="${args.pageable}" 
      [sortable]="${args.sortable}" 
      [navigable]="${args.navigable}"
      [groupable]="${args.groupable}"
      [reorderable]="${args.reorderable}"
      [resizable]="${args.resizable}"
      [mode]= "'${args.mode}'"
      [checkBoxWidth]="'${args.checkBoxWidth}'"
      [checkBoxColumnMenu]="${args.checkBoxColumnMenu}"
      [height]="'${args.height}'"
      [gridRowSelectByValue]="'${args.gridRowSelectByValue}'"
      [checkBoxColumnShowSelectedAll]="${args.checkBoxColumnShowSelectedAll}"
      [checkBoxRequired]="${args.checkBoxRequired}"
      [noRecordsContent]="'${args.noRecordsContent}'"
      [showToolbar]="${args.showToolbar}"
      [kebabList]="kebabList"
      [showKebab]="${args.showKebab}"
      [dragColumns]="${args.dragColumns}"
      kebabColumnWidth="${args.kebabColumnWidth}"
      [preSelected]="preSelected"
      [ignoreColumnCellClick]="ignoreColumnCellClick"
      [allowColumnClick]="${args.allowColumnClick}"
      [expandRowIndex]="expandRowIndex"
      [customExpandRowColumn]="${args.customExpandRowColumn}"
      [deleteColumnWidth]="${args.deleteColumnWidth}"
      [expandRowColumnWidth]="${args.expandRowColumnWidth}"
      >
      <input placeholder="Search in all columns..." kendoTextBox /> 
      <lib-grid-spacer></lib-grid-spacer>
      <lib-button label="Add Profile" themeColor="primary"></lib-button>
      <lib-grid-spacer></lib-grid-spacer>
      <lib-button label="Edit Profile" themeColor="secondary"></lib-button>
  
        <ng-template let-dataItem #multiColumnData>
        {{dataItem.ProductName}} - {{dataItem.ProductID}}
        </ng-template>
  
        <ng-template let-dataItem #impactColumn>
          {{dataItem.ProductName}}
        </ng-template>
  
        <ng-template let-dataItem #detailTemplate>
          <section>
            <p><strong>In Stock:</strong> {{ dataItem.expandedData.UnitsInStock }} units</p>
            <p><strong>On Order:</strong> {{ dataItem.expandedData.UnitsOnOrder }} units</p>
            <p>
              <strong>Reorder Level:</strong> {{ dataItem.expandedData.ReorderLevel }} units
            </p>
            <p><strong>Discontinued:</strong> {{ dataItem.expandedData.Discontinued }}</p>
            <p>
              <strong>Category:</strong> {{ dataItem.expandedData.Category?.CategoryName }} -
              {{ dataItem.expandedData.Category?.Description }}
            </p>
          </section>
        </ng-template>
  
      </lib-grid>
  
    `,
  }),
};

export default meta;

type Story = StoryObj<LibGridComponent>;

export const Grid: Story = {
  args: {
    gridData: [
      {
        ProductID: 1,
        ProductName: 'Chai',
        UnitPrice: 18,
        Category: {
          CategoryID: 1,
          CategoryName: 'Beverages',
        },
        expandedData: {
          UnitsInStock: '1',
          UnitsOnOrder: '1',
          ReorderLevel: 'RL 1',
          Discontinued: 'True',
          Category: {
            CategoryName: 'Cat 1',
            Description: 'Cat Desc 1',
          },
        },
      },
      {
        ProductID: 2,
        ProductName: 'Chang',
        UnitPrice: 19,
        Category: {
          CategoryID: 1,
          CategoryName: 'Beverages',
        },
        expandedData: {
          UnitsInStock: '2',
          UnitsOnOrder: '2',
          ReorderLevel: 'RL 2',
          Discontinued: 'False',
          Category: {
            CategoryName: 'Cat 2',
            Description: 'Cat Desc 2',
          },
        },
      },
      {
        ProductID: 3,
        ProductName: 'Aniseed Syrup',
        UnitPrice: 10,
        Category: {
          CategoryID: 2,
          CategoryName: 'Condiments',
        },
        expandedData: {
          UnitsInStock: '3',
          UnitsOnOrder: '3',
          ReorderLevel: 'RL 3',
          Discontinued: 'True',
          Category: {
            CategoryName: 'Cat 3',
            Description: 'Cat Desc 3',
          },
        },
      },
      {
        ProductID: 4,
        ProductName: 'Chai',
        UnitPrice: 18,
        Category: {
          CategoryID: 1,
          CategoryName: 'Beverages',
        },
        expandedData: {
          UnitsInStock: '1',
          UnitsOnOrder: '1',
          ReorderLevel: 'RL 1',
          Discontinued: 'True',
          Category: {
            CategoryName: 'Cat 1',
            Description: 'Cat Desc 1',
          },
        },
      },
      {
        ProductID: 5,
        ProductName: 'Chang',
        UnitPrice: 19,
        Category: {
          CategoryID: 1,
          CategoryName: 'Beverages',
        },
        expandedData: {
          UnitsInStock: '2',
          UnitsOnOrder: '2',
          ReorderLevel: 'RL 2',
          Discontinued: 'False',
          Category: {
            CategoryName: 'Cat 2',
            Description: 'Cat Desc 2',
          },
        },
      },
      {
        ProductID: 6,
        ProductName: 'Aniseed Syrup',
        UnitPrice: 10,
        Category: {
          CategoryID: 2,
          CategoryName: 'Condiments',
        },
        expandedData: {
          UnitsInStock: '3',
          UnitsOnOrder: '3',
          ReorderLevel: 'RL 3',
          Discontinued: 'True',
          Category: {
            CategoryName: 'Cat 3',
            Description: 'Cat Desc 3',
          },
        },
      },{
        ProductID: 7,
        ProductName: 'Chai',
        UnitPrice: 18,
        Category: {
          CategoryID: 1,
          CategoryName: 'Beverages',
        },
        expandedData: {
          UnitsInStock: '1',
          UnitsOnOrder: '1',
          ReorderLevel: 'RL 1',
          Discontinued: 'True',
          Category: {
            CategoryName: 'Cat 1',
            Description: 'Cat Desc 1',
          },
        },
      },
      {
        ProductID: 8,
        ProductName: 'Chang',
        UnitPrice: 19,
        Category: {
          CategoryID: 1,
          CategoryName: 'Beverages',
        },
        expandedData: {
          UnitsInStock: '2',
          UnitsOnOrder: '2',
          ReorderLevel: 'RL 2',
          Discontinued: 'False',
          Category: {
            CategoryName: 'Cat 2',
            Description: 'Cat Desc 2',
          },
        },
      },
      {
        ProductID: 9,
        ProductName: 'Aniseed Syrup',
        UnitPrice: 10,
        Category: {
          CategoryID: 2,
          CategoryName: 'Condiments',
        },
        expandedData: {
          UnitsInStock: '3',
          UnitsOnOrder: '3',
          ReorderLevel: 'RL 3',
          Discontinued: 'True',
          Category: {
            CategoryName: 'Cat 3',
            Description: 'Cat Desc 3',
          },
        },
      },{
        ProductID: 10,
        ProductName: 'Chai',
        UnitPrice: 18,
        Category: {
          CategoryID: 1,
          CategoryName: 'Beverages',
        },
        expandedData: {
          UnitsInStock: '1',
          UnitsOnOrder: '1',
          ReorderLevel: 'RL 1',
          Discontinued: 'True',
          Category: {
            CategoryName: 'Cat 1',
            Description: 'Cat Desc 1',
          },
        },
      },
      {
        ProductID: 11,
        ProductName: 'Chang',
        UnitPrice: 19,
        Category: {
          CategoryID: 1,
          CategoryName: 'Beverages',
        },
        expandedData: {
          UnitsInStock: '2',
          UnitsOnOrder: '2',
          ReorderLevel: 'RL 2',
          Discontinued: 'False',
          Category: {
            CategoryName: 'Cat 2',
            Description: 'Cat Desc 2',
          },
        },
      },
      {
        ProductID: 12,
        ProductName: 'Aniseed Syrup',
        UnitPrice: 10,
        Category: {
          CategoryID: 2,
          CategoryName: 'Condiments',
        },
        expandedData: {
          UnitsInStock: '3',
          UnitsOnOrder: '3',
          ReorderLevel: 'RL 3',
          Discontinued: 'True',
          Category: {
            CategoryName: 'Cat 3',
            Description: 'Cat Desc 3',
          },
        },
      },
    ],
    columnData: [
      {
        field: 'Control',
        title: 'Control',
        custom: true,
        columnWidth: '300',
      },
      {
        field: 'UnitPrice',
        title: 'UnitPrice',
        impact: true,
        columnWidth: '100',
      },
      {
        field: 'Category.CategoryID',
        title: 'CategoryID',
        columnWidth: '200',
      },
      {
        field: 'Category.CategoryName',
        title: 'CategoryName',
        columnWidth: '200',
      },
    ],
    pageSize: 3,
    pageable: true,
    sortable: true,
    groupable: true,
    reorderable: true,
    resizable: true,
    checkBoxColumnMenu: false,
    checkBoxColumnShowSelectedAll: true,
    checkBoxRequired: true,
    dragColumns: true,
    mode: 'multiple',
    gridRowSelectByValue: 'ProductID',
    navigable: true,
    noRecordsContent: 'No Integrations Added yet',
    showToolbar: true,
    kebabList: [
      {
        text: 'Download',
      },
    ],
    icon: moreVerticalIcon,
    showKebab: true,
    kebabColumnWidth: 80,
    preSelected: [1, 2],
    ignoreColumnCellClick: [0, 5],
    allowColumnClick: true,
    customExpandRowColumn:true,
    expandRowIndex:[0],
    checkBoxWidth:28,
    deleteColumnWidth:56,
    expandRowColumnWidth:42
  },
};

export const gridInteraction: Story = {
  args: {
    gridData: [
      {
        ProductID: 1,
        ProductName: 'Chai',
        UnitPrice: 18,
        Category: {
          CategoryID: 1,
          CategoryName: 'Beverages',
        },
      },
      {
        ProductID: 2,
        ProductName: 'Chang',
        UnitPrice: 19,
        Category: {
          CategoryID: 1,
          CategoryName: 'Beverages',
        },
      },
      {
        ProductID: 3,
        ProductName: 'Aniseed Syrup',
        UnitPrice: 10,
        Category: {
          CategoryID: 2,
          CategoryName: 'Condiments',
        },
      },
    ],
    columnData: [
      {
        field: 'ProductID',
        title: 'ProductID',
      },
      {
        field: 'UnitPrice',
        title: 'UnitPrice',
      },
      {
        field: 'Category.CategoryID',
        title: 'CategoryID',
      },
      {
        field: 'Category.CategoryName',
        title: 'CategoryName',
      },
    ],
    pageSize: 0,
    pageable: false,
    sortable: true,
    groupable: false,
    reorderable: false,
    resizable: true,
    checkBoxColumnMenu: false,
    checkBoxColumnShowSelectedAll: true,
    checkBoxRequired: true,
    gridRowSelectByValue: 'ProductID',
    mode: 'multiple',
    navigable: true,
  },
  play: async ({ canvasElement }) => {
    const canvas = within(canvasElement);
    const checkShow = await canvas.getAllByRole('checkbox');
    await userEvent.click(checkShow[0]);
  },
};
