import { APP_INITIALIZER } from '@angular/core';
import '@angular/localize/init';
import { provideAnimations } from '@angular/platform-browser/animations';
import { TooltipModule } from '@components/atoms/tooltip/tooltip.module';
import { TypographyModule } from '@components/atoms/typography/typography.module';
import { SidebarComponent } from '@components/molecules/sidebar/sidebar.component';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import {
  moduleMetadata,
  Meta,
  StoryObj,
  applicationConfig,
} from '@storybook/angular';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<SidebarComponent> = {
  title: 'Molecules/Sidebar Component',
  component: SidebarComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      declarations: [SidebarComponent],
      imports: [TooltipModule, TypographyModule, ThemesModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    isSidebarOpen: {
      options: [true, false],
      control: 'radio',
    },
    sideNavbarMenus: {},
  },
  render: (args: SidebarComponent) => ({
    props: args,
  }),
};

export default meta;

type Story = StoryObj<SidebarComponent>;

export const Sidebar: Story = {
  args: {
    isSidebarOpen: true,
    sideNavbarMenus: [
      {
        name: 'Report',
        route: '/compliance/scan-result',
        active: false,
        authorized: true,
        visible: true,
        openInNewPage: false,
        icon: 'ad_units',
      },
      {
        name: 'Scan Job',
        route: '/compliance',
        active: true,
        authorized: true,
        visible: true,
        openInNewPage: false,
        icon: 'document_scanner',
      },
      {
        name: 'Profile',
        route: '/compliance/scan-details',
        active: false,
        authorized: true,
        visible: true,
        openInNewPage: false,
        icon: 'task',
      },
    ],
  },
};
