import '@angular/localize/init';
import {
  moduleMetadata,
  StoryObj,
  Meta,
  applicationConfig,
} from '@storybook/angular';
import { TabstripModule } from '@components/molecules/tabstrip/tabstrip.module';
import { TabstripComponent } from '@components/molecules/tabstrip/tabstrip.component';
import { LayoutModule } from '@progress/kendo-angular-layout';
import { APP_INITIALIZER } from '@angular/core';
import { ThemesModule, ThemesService } from '@progress-chef/interact-themes';
import { userEvent, within } from '@storybook/test';
import { provideAnimations } from '@angular/platform-browser/animations';

function themesServiceFactory(themesService: ThemesService) {
  return () => themesService;
}

const meta: Meta<TabstripComponent> = {
  title: 'Molecules/Tabstrip Component',
  component: TabstripComponent,
  decorators: [
    applicationConfig({
      providers: [provideAnimations()],
    }),
    moduleMetadata({
      imports: [ThemesModule, TabstripModule, LayoutModule],
      providers: [
        {
          provide: APP_INITIALIZER,
          useFactory: themesServiceFactory,
          deps: [ThemesService],
        },
      ],
    }),
  ],
  tags: ['autodocs'],
  argTypes: {
    height: {
      control: 'text',
    },
    alignment: {
      options: ['start', 'end', 'center', 'justify'],
      control: 'radio',
    },
    position: {
      options: ['top', 'left', 'right', 'bottom'],
      control: 'radio',
    },
    onTabSelect: {
      action: true,
    },
    selectedTab: {
      control : 'number'
    }
  },
  render: (args: TabstripComponent) => ({
    props: args,
    template: `
      <div class="wrapper">
        <lib-tabstrip [position]="'${args.position}'" [alignment]="'${args.alignment}'" [height]="'${args.height}'" [selectedTab]="'${args.selectedTab}'">
          <lib-tabstrip-tab title="tab1" [customHeader]="true">
            <ng-template #titleRef>
              <span>Custom Header</span>
            </ng-template>
            <ng-template #contentRef>
              <div class="content">
                <img src="https://www.telerik.com/kendo-angular-ui-develop/components/layout/assets/tabstrip/rainy.png" />
                <h2>17<span>ºC</span></h2>
                <span>Rainy weather in Paris.</span>
              </div>
            </ng-template>
          </lib-tabstrip-tab>
          <lib-tabstrip-tab title="tab2" [customHeader]="false">
            <ng-template #titleRef>
              <span>Title 2 </span>
              <span>Count</span>
            </ng-template>
            <ng-template #contentRef>
              <div class="content">
                <img src="https://www.telerik.com/kendo-angular-ui-develop/components/layout/assets/tabstrip/cloudy.png" />
                <h2>17<span>ºC</span></h2>
                <span>Rainy weather in Bangalore.</span>
              </div>
            </ng-template>
          </lib-tabstrip-tab>
          <lib-tabstrip-tab title="tab3">
          <header class="tabstriptitle-2">tab 13</header>
            <ng-template #titleRef>
              <span>Title 3 </span>
              <span>Count</span>
            </ng-template>
            <ng-template #contentRef>
              <div class="content">
                <img src="https://www.telerik.com/kendo-angular-ui-develop/components/layout/assets/tabstrip/sunny.png" />
                <h2>17<span>ºC</span></h2>
                <span>Rainy weather in China.</span>
              </div>
            </ng-template>
          </lib-tabstrip-tab>
        </lib-tabstrip>
      </div>
      `,
  }),
};

export default meta;

type Story = StoryObj<TabstripComponent>;

export const Tabstrip: Story = {
  args: {
    height: '82',
    alignment: 'justify',
    position: 'top',
    selectedTab : 0
  },
};

Tabstrip.play = async ({ canvasElement }) => {
  const canvas = within(canvasElement);
  const tab = await canvas.getByRole('tablist');
  await userEvent.click(tab);
};
